@@ -3,9 +3,11 @@ import random
 import time
 import sys
 import signal
-import numpy as np
 import logging
-import asyncio
+import os
+from copy import deepcopy
+from multiprocessing import Process, Queue
+from functools import wraps
 
 logging.basicConfig(level=logging.DEBUG, filename='system.log')
 
@@ -17,6 +19,19 @@ kannada = ['ಅ', 'ಆ', 'ಇ', 'ಈ', 'ಎ', 'ಏ', 'ಐ', 'ಒ', 'ಓ', 'ಔ',
 
 languages = english + kannada + sanskrit + greek + numbers
 
+class TerminalResize(Exception):
+    pass
+
+
+def process_ehandler(func):
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        try:
+            return func(*args, **kwargs)
+        except KeyboardInterrupt:
+            sys.exit(0)
+    return wrapper
+
 class Bar(object):
 
     def __init__(self, t, x):
@@ -36,11 +51,11 @@ class Bar(object):
         if self.has_fallen(): return
 
         if self.pos < self.length:
-            scene[self.pos-1, self.x] = go_green(scene[self.pos-1, self.x])
+            scene[self.pos-1][self.x] = go_green(scene[self.pos-1][self.x])
             scene[self.pos][self.x] = random.choice(self.languages)
         else:
             if self.pos < self.t.height:
-                scene[self.pos-1, self.x] = go_green(scene[self.pos-1, self.x])
+                scene[self.pos-1][self.x] = go_green(scene[self.pos-1][self.x])
                 scene[self.pos][self.x] = random.choice(self.languages)
             if self.pos - self.length < self.t.height:
                 scene[self.pos-self.length][self.x] = ' '
@@ -65,17 +80,28 @@ def worker(bars, scene, columns, idx, t):
         if b.has_fallen():
             columns[idx].pop(0)
 
-#@profile
-async def matrix_ns(t: Terminal):
+@process_ehandler
+def print_scene(q):
+    t = Terminal()
+    while True:
+        _scene = q.get()
+        with t.location(0, 0):
+            print('\n'.join([''.join(l) for l in _scene]), end='\r')
+        time.sleep(1/30)
+
+
+@process_ehandler
+def matrix_ns(q):
     """
       Matrix new style scrolling
-    """
-    scene = np.array([[' ' for x in range(t.width)] for y in range(t.height)], dtype=object)
-    columns = [random.choice([[Bar(t, x)]]+[None]*3)
-               if x % 2 else None for x in range(t.width)]
+    """    
+    t = Terminal()
+    scene = [[' ' for x in range(t.width)] for y in range(t.height)]
+    columns = [random.choice([[Bar(t, x)]]+[None]*3) if x%2 else None for x in range(t.width)]
 
     with t.hidden_cursor():
         while True:
+            if q.full(): continue
             for idx, bars in enumerate(columns):
                 if idx % 2: continue
                 if not bars:
@@ -83,18 +109,32 @@ async def matrix_ns(t: Terminal):
                             [[Bar(t, idx)]] + [None]*3)
                     continue
                 worker(bars, scene, columns, idx, t)
-            with t.location(0, 0):
-                print('\n'.join([''.join(line) for line in scene]), end='\r')
-            await asyncio.sleep(0.03)
+            q.put(scene)
 
 def main():
     try:
+        q = Queue(1)
         t = Terminal()
+
+        def handle_sighup(signal, frame):
+            raise TerminalResize
+
+        signal.signal(signal.SIGWINCH, handle_sighup)
+
         print(t.clear())
-        asyncio.run(matrix_ns(t))
+        procs = [
+                    Process(target=print_scene, args=(q,), daemon=True),
+                    Process(target=matrix_ns, args=(q,), daemon=True)
+                ]
+        for p in procs: p.start()
+        for p in procs: p.join()
     except KeyboardInterrupt:
         print(t.clear())
         sys.exit(0)
+    except TerminalResize:
+        for p in procs: p.terminate()
+        os.execv(sys.executable, ['python'] + sys.argv)
+
 
 
 if __name__ == "__main__":
